//
//  SNCreateAppViewController.m
//  Snap
//
//  Created by James Martinez on 1/2/13.
//  Copyright (c) 2013 James Martinez. All rights reserved.
//

#import "SNCreateAppViewController.h"
#import "NSImage+Snap.h"
#import "NSError+Snap.h"
#import "SNAlert.h"

static NSString *const chromePath = @"/Applications/Google Chrome.app/Contents/MacOS/Google Chrome";
static NSString *const applicationsPath = @"/Applications";

@interface SNCreateAppViewController ()
@property(weak, nonatomic) IBOutlet NSTextField *appName;
@property(weak, nonatomic) IBOutlet NSTextField *appURL;
@property(weak, nonatomic) IBOutlet NSImageView *appIcon;

@property(strong, nonatomic) NSString *iconPath;
@property(strong, nonatomic) NSString *resourceDirectory;
@property(strong, nonatomic) NSString *execDirectory;
@property(strong, nonatomic) NSString *profileDirectory;
@property(strong, nonatomic) NSString *contentsDirectory;
@end

@implementation SNCreateAppViewController

#pragma mark - View Actions

- (IBAction)browseForIcon:(id)sender {
    NSArray *fileTypesArray = [NSArray arrayWithObjects:@"jpg", @"gif", @"png", @"tiff", nil];

    NSOpenPanel *openPanel = [NSOpenPanel openPanel];
    [openPanel setCanChooseFiles:YES];
    [openPanel setAllowedFileTypes:fileTypesArray];
    [openPanel setAllowsMultipleSelection:FALSE];

    if ([openPanel runModal] == NSOKButton && openPanel.URL.path.length != 0) {
        NSImage *iconImage = [[NSImage alloc] initWithContentsOfFile:openPanel.URL.path];
        if (iconImage.size.height == iconImage.size.width) {
            self.iconPath = openPanel.URL.path;
            self.appIcon.image = iconImage;
        } else {
            [[SNAlert snapErrorAlertWithMessage:@"The image you provided is not square."] runModal];
            self.appIcon.image = nil;
        }
    }
}

- (IBAction)finish:(id)sender {
    if (self.appName.stringValue.length == 0) {
        [[SNAlert snapErrorAlertWithMessage:@"Invalid application name."] runModal];
        return;
    } else if (self.appURL.stringValue.length == 0) {
        [[SNAlert snapErrorAlertWithMessage:@"Invalid application URL."] runModal];
        return;
    }

    self.resourceDirectory = [NSString stringWithFormat:@"%@/%@.app/Contents/Resources", applicationsPath, self.appName.stringValue];
    self.execDirectory = [NSString stringWithFormat:@"%@/%@.app/Contents/MacOS", applicationsPath, self.appName.stringValue];
    self.profileDirectory = [NSString stringWithFormat:@"%@/%@.app/Contents/Profile", applicationsPath, self.appName.stringValue];
    self.contentsDirectory = [NSString stringWithFormat:@"%@/%@.app/Contents", applicationsPath, self.appName.stringValue];

    NSArray *directories = @[self.resourceDirectory, self.execDirectory, self.profileDirectory];
    NSError *error = nil;
    if ([self makeNewAppDirectoriesWithDirectories:directories error:&error]) {
        NSLog(@"New app directories successfully created.");
    } else {
        [[SNAlert snapErrorAlertWithMessage:error.localizedDescription] runModal];
        return;
    }
    if (self.iconPath.length != 0) {
        if ([self convertImagesWithSourceImagePath:self.iconPath andDestinationImagesDirectory:self.resourceDirectory error:&error]) {
            NSLog(@"Images successfully converted.");
        } else {
            [[SNAlert snapErrorAlertWithMessage:error.localizedDescription] runModal];
            return;
        }
    }
    if ([self createPlistFileWithDestinationDirectory:self.contentsDirectory usingAppName:self.appName.stringValue error:&error]) {
        NSLog(@"Plist successfully created.");
    } else {
        [[SNAlert snapErrorAlertWithMessage:error.localizedDescription] runModal];
        return;
    }
    if ([self createExecutableWithDestinationDirectory:self.execDirectory usingAppUrl:self.appURL.stringValue andProfileDirectory:self.profileDirectory error:&error]) {
        NSLog(@"Executable successfully created.");
    } else {
        [[SNAlert snapErrorAlertWithMessage:error.localizedDescription] runModal];
        return;
    }
    [[SNAlert snapSuccessAlertWithMessage:@"Your Snap App was created and placed into your 'Applications' folder."] runModal];
}

- (BOOL)makeNewAppDirectoriesWithDirectories:(NSArray *)directories error:(NSError **)error {
    NSFileManager *fileManager = [NSFileManager defaultManager];
    for (NSString *directory in directories) {
        if (![fileManager fileExistsAtPath:directory]) {
            if (![fileManager createDirectoryAtPath:directory withIntermediateDirectories:YES attributes:nil error:error]) {
                NSLog(@"Error creating new directory.");
                return NO;
            } else {
                NSLog(@"New directory created successfully.");
            }
        } else {
            NSLog(@"File already exists at that path.");
            *error = [NSError snapErrorWithCode:SNErrorDuplicateFileExists localizedDescription:@"Application already exists with that name."];
            return NO;
        }
    }
    return YES;
}

- (BOOL)convertImagesWithSourceImagePath:(NSString *)imagePath andDestinationImagesDirectory:(NSString *)destinationDirectory  error:(NSError **)error {
    NSImage *originalImage = [[NSImage alloc] initWithContentsOfFile:imagePath];
    NSImage *resizedImage = [NSImage resizeImage:originalImage toSize:CGSizeMake(128, 128)];

    NSURL *fileURL = [NSURL fileURLWithPath:[NSString stringWithFormat:@"%@/icon.icns", destinationDirectory]];
    CGImageRef imageRef = [self refFromImage:resizedImage];
    CGImageDestinationRef imageDestinationRef = CGImageDestinationCreateWithURL((__bridge CFURLRef) fileURL, kUTTypeAppleICNS, 1, NULL);
    CGImageDestinationAddImage(imageDestinationRef, imageRef, NULL);
    if (CGImageDestinationFinalize(imageDestinationRef)) {
        NSLog(@"icon.icns created successfully.");
    } else {
        NSLog(@"Error writing to icon.icns.");
        *error = [NSError snapErrorWithCode:SNErrorImageConversionFailed localizedDescription:@"Error creating icon."];
        return NO;
    }
    CFRelease(imageDestinationRef);
    return YES;
}

- (BOOL)createPlistFileWithDestinationDirectory:(NSString *)destinationDirectory usingAppName:(NSString *)appName  error:(NSError **)error {
    NSMutableDictionary *plistDict = [NSMutableDictionary new];
    [plistDict setObject:appName forKey:@"CFBundleExecutable"];
    [plistDict setObject:@"icon" forKey:@"CFBundleIconFile"];
    NSData *plist =  [NSPropertyListSerialization dataWithPropertyList:(id)plistDict
                                                                format:NSPropertyListXMLFormat_v1_0
                                                               options:0
                                                                 error:error];
    if (plist == nil) {
        return NO;
    }

    NSString *plistPath = [NSString stringWithFormat:@"%@/Info.plist", destinationDirectory];
    if ([plist writeToFile:plistPath options:NSDataWritingAtomic error:error]) {
        NSLog(@"Plist created successfully.");
    } else {
        NSLog(@"Error creating Plist.");
        return NO;
    }
    return YES;
}

- (BOOL)createExecutableWithDestinationDirectory:(NSString *)destinationDirectory usingAppUrl:(NSString *)appURL andProfileDirectory:(NSString *)profileDirectory  error:(NSError **)error {
    NSString *escapedChromePath = [chromePath stringByReplacingOccurrencesOfString:@" " withString:@"\\ "];
    NSString *executableString = [NSString stringWithFormat:@"#!/bin/sh\nexec %@ --app=\"%@\" --user-data-dir=\"%@\" \"$@\"", escapedChromePath, appURL, profileDirectory];
    NSString *destinationFile = [NSString stringWithFormat:@"%@/%@", destinationDirectory, self.appName.stringValue];
    if ([[executableString dataUsingEncoding:NSUTF8StringEncoding] writeToFile:destinationFile options:NSDataWritingAtomic error:error]) {
        NSLog(@"Executable created successfully.");
    } else {
        NSLog(@"Error creating executable.");
        return NO;
    }

    NSMutableDictionary *permissionsDictionary = [[NSMutableDictionary alloc] init];
    [permissionsDictionary setObject:[NSNumber numberWithInt:511] forKey:NSFilePosixPermissions]; //511 is decimal for the 777 octal

    if ([[NSFileManager defaultManager] setAttributes:permissionsDictionary ofItemAtPath:destinationFile error:error]) {
        NSLog(@"Permissions changed successfully.");
    } else {
        NSLog(@"Error changing permissions.");
        return NO;
    }
    return YES;
}

#pragma mark - Helpers

- (CGImageRef)refFromImage:(NSImage *)image {
    CGImageSourceRef source;
    source = CGImageSourceCreateWithData((__bridge CFDataRef) [image TIFFRepresentation], NULL);
    CGImageRef maskRef = CGImageSourceCreateImageAtIndex(source, 0, NULL);
    return maskRef;
}

@end